@@ -9,7 +9,7 @@ from sklearn.metrics import mean_squared_error
 from sklearn.utils import check_array, check_consistent_length, check_X_y
 from sklearn.utils.validation import check_is_fitted, check_scalar
 
-from datafold.decorators import warn_experimental_class, warn_known_bug
+from datafold._decorators import warn_known_bug
 from datafold.dynfold.dmap import _DmapKernelAlgorithms
 from datafold.pcfold import PCManifold
 from datafold.pcfold.distance import compute_distance_matrix
@@ -17,7 +17,7 @@ from datafold.pcfold.kernels import DmapKernelFixed, GaussianKernel, PCManifoldK
 from datafold.utils.general import mat_dot_diagmat
 
 
-class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMixin):
+class GeometricHarmonicsInterpolator(RegressorMixin, MultiOutputMixin, BaseEstimator):
     """Interpolation of function values on high dimensional data with manifold assumption.
 
     Parameters
@@ -25,7 +25,7 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
     kernel
         Internal kernel to describe proximity between points. The kernel is passed
         as an `internal_kernel` to :class:`.DmapKernelFixed`, which describes
-        the diffusion process.
+        the diffusion process. Defaults to :py:class:`.GaussianKernel` with bandwidth 1.0.
 
     n_eigenpairs
         Number of eigenpairs to compute from kernel matrix.
@@ -83,7 +83,8 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
 
     def __init__(
         self,
-        kernel: PCManifoldKernel = GaussianKernel(epsilon=1.0),
+        kernel: Optional[PCManifoldKernel] = None,
+        *,  # keyword-only
         n_eigenpairs: int = 10,
         is_stochastic: bool = False,
         alpha: float = 1,
@@ -113,18 +114,13 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
             self.eigenvectors_, np.reciprocal(self.eigenvalues_)
         ) @ (self.eigenvectors_.T @ self.y_)
 
-    def _validate(
-        self, X: np.ndarray, y: np.ndarray = None, ensure_min_samples=1
-    ) -> Union[np.ndarray, np.ndarray]:
-
-        check_consistent_length(X, y)
-
+    def _validate_kwargs(self, X, ensure_min_samples, during_fit):
         if isinstance(X, np.memmap):
             copy = True
         else:
             copy = False
 
-        kwargs = {
+        validate_kwargs = {
             "accept_sparse": False,
             "copy": copy,
             "accept_large_sparse": False,
@@ -136,23 +132,26 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
             "ensure_min_features": 1,
         }
 
-        if y is None:
-            X = check_array(X, **kwargs)
-        else:
-            if isinstance(y, np.ndarray) and y.ndim == 1:
-                y = y[:, np.newaxis]
+        if during_fit:
+            validate_kwargs["multi_output"] = True
+            validate_kwargs["y_numeric"] = True
 
-            kwargs["multi_output"] = True
-            kwargs["y_numeric"] = True
-            X, y = check_X_y(X, y, **kwargs)
+        return validate_kwargs
 
-        return X, y
+    def _more_tags(self):
+        # poor_score=True disables a regression test on a Boston Housing dataset
+        # see "check_regressors_train" in sklearn/estimator_checks.py
+        # GHI computes the negated root mean squared error and therefore would
+        # always fail because of this assert in the test:
+        #     regressor.score(X, y_) > 0.5
+        return {"requires_y": True, "multioutput": True, "poor_score": True}
 
     def _get_tags(self):
-        # _tags = super(GeometricHarmonicsInterpolator, self)._get_tags()
-        self._more_tags()["multioutput"] = True
         return super(GeometricHarmonicsInterpolator, self)._get_tags()
 
+    def _get_default_kernel(self):
+        return GaussianKernel(epsilon=1.0)
+
     def _setup_default_dist_kwargs(self):
         from copy import deepcopy
 
@@ -161,31 +160,6 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
         self.dist_kwargs_.setdefault("kmin", 0)
         self.dist_kwargs_.setdefault("backend", "guess_optimal")
 
-    def predict(self, X: np.ndarray) -> np.ndarray:
-        """Evaluate model for out-of-sample points.
-
-        Parameters
-        ----------
-        X
-            Points of shape `(n_samples, n_features)`.
-
-        Returns
-        -------
-        numpy.ndarray
-            The interpolated function values of shape `(n_samples, n_targets)`.
-        """
-
-        check_is_fitted(self)
-
-        X, _ = self._validate(X, ensure_min_samples=1)
-
-        kernel_output = self.X_.compute_kernel_matrix(Y=X, **self._cdist_kwargs)
-        kernel_matrix_, _, _ = PCManifoldKernel.read_kernel_output(
-            kernel_output=kernel_output
-        )
-
-        return np.squeeze(kernel_matrix_ @ self._aux)
-
     def fit(
         self, X: np.ndarray, y: np.ndarray, store_kernel_matrix: bool = False
     ) -> "GeometricHarmonicsInterpolator":
@@ -207,26 +181,44 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
         GeometricHarmonicsInterpolator
             self
         """
-        X, y = self._validate(X, y=y, ensure_min_samples=2)
+
+        # function provided by sklearn
+        # internally sets attribute n_features_in_
+        X, y = self._validate_data(
+            X=X,
+            y=y,
+            reset=True,
+            validate_separately=False,
+            **self._validate_kwargs(X, ensure_min_samples=2, during_fit=True),
+        )
+        if isinstance(y, np.ndarray) and y.ndim == 1:
+            y = y[:, np.newaxis]
+        check_consistent_length(X, y)
 
         self._setup_default_dist_kwargs()
 
+        internal_kernel = (
+            self.kernel if self.kernel is not None else self._get_default_kernel()
+        )
+
         self._dmap_kernel = DmapKernelFixed(
-            internal_kernel=self.kernel,
+            internal_kernel=internal_kernel,
             is_stochastic=self.is_stochastic,
             alpha=self.alpha,
             symmetrize_kernel=self.symmetrize_kernel,
         )
 
         self.X_ = PCManifold(
-            X, kernel=self._dmap_kernel, dist_kwargs=self.dist_kwargs_,
+            X,
+            kernel=self._dmap_kernel,
+            dist_kwargs=self.dist_kwargs_,
         )
         self.y_ = y
 
         check_scalar(
             self.n_eigenpairs,
             "n_eigenpairs",
-            target_type=(np.integer, int),
+            target_type=(int, np.integer),
             min_val=1,
             max_val=self.X_.shape[0] - 1,
         )
@@ -263,6 +255,32 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
 
         return self
 
+    def predict(self, X: np.ndarray) -> np.ndarray:
+        """Evaluate model for out-of-sample points.
+
+        Parameters
+        ----------
+        X
+            Points of shape `(n_samples, n_features)`.
+
+        Returns
+        -------
+        numpy.ndarray
+            The interpolated function values of shape `(n_samples, n_targets)`.
+        """
+
+        check_is_fitted(self)
+        X = check_array(
+            X, **self._validate_kwargs(X, ensure_min_samples=1, during_fit=False)
+        )
+
+        kernel_output = self.X_.compute_kernel_matrix(Y=X, **self._cdist_kwargs)
+        kernel_matrix_, _, _ = PCManifoldKernel.read_kernel_output(
+            kernel_output=kernel_output
+        )
+
+        return np.squeeze(kernel_matrix_ @ self._aux)
+
     @warn_known_bug(gitlab_issue=16)
     def gradient(self, X: np.ndarray, vcol: Optional[int] = None) -> np.ndarray:
         """Evaluate gradient of interpolator at the given points.
@@ -275,7 +293,7 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
         ----------
         X
             Out-of-sample points to compute the gradient at.
-            
+
         vcol
             Column index of the corresponding function value to compute the gradient of.
             Has to be given for multivariate interpolation.
@@ -289,7 +307,11 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
         # TODO: generalize to all columns (if required...). Note that this will be a
         #  tensor then.
 
-        X, _ = self._validate(X, ensure_min_samples=1)
+        from sklearn.utils.validation import check_array
+
+        X = check_array(
+            X, self._validate_kwargs(X, ensure_min_samples=1, during_fit=False)
+        )
 
         assert self.X_ is not None and self.y_ is not None  # prevents mypy warnings
 
@@ -376,7 +398,15 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
         float
             score
         """
-        X, y = self._validate(X, y=y, ensure_min_samples=1)
+        # during_fit enforces the check on y
+        X, y = check_X_y(
+            X, y=y, **self._validate_kwargs(X, ensure_min_samples=1, during_fit=True)
+        )
+
+        if isinstance(y, np.ndarray) and y.ndim == 1:
+            y = y[:, np.newaxis]
+        check_consistent_length(X, y)
+
         y_pred = self.predict(X)
 
         score = mean_squared_error(
@@ -393,7 +423,9 @@ class GeometricHarmonicsInterpolator(BaseEstimator, RegressorMixin, MultiOutputM
 
 
 @NotImplementedError
-class MultiScaleGeometricHarmonicsInterpolator(GeometricHarmonicsInterpolator):
+class MultiScaleGeometricHarmonicsInterpolator(
+    GeometricHarmonicsInterpolator
+):  # pragma: no cover
     """
     .. warning::
         This class is not documented and in experimental state. Contributions are welcome:
@@ -436,10 +468,11 @@ class MultiScaleGeometricHarmonicsInterpolator(GeometricHarmonicsInterpolator):
         scale = self.initial_scale
         error_not_in_range = True
 
+        from scipy.sparse.linalg import eigsh
+
         from datafold.pcfold import PCManifold
         from datafold.pcfold.kernels import GaussianKernel
         from datafold.utils.general import diagmat_dot_mat, sort_eigenpairs
-        from scipy.sparse.linalg import eigsh
 
         mu_l_ = None
         phi_l_ = None
@@ -481,7 +514,7 @@ class MultiScaleGeometricHarmonicsInterpolator(GeometricHarmonicsInterpolator):
         self._aux = phi_l_ @ diagmat_dot_mat(np.reciprocal(mu_l_), phi_l_.T) @ y
 
     def __call__(self, X):
-        X, _ = self._validate(X=X, y=None, ensure_min_samples=1)
+        X, _ = self._validate(X=X, y=None, during_fit=False, ensure_min_samples=1)
 
         kernel_matrix = self.X.compute_kernel_matrix(
             Y=X, row_sums_alpha_fit=self._row_sums_alpha
@@ -492,7 +525,7 @@ class MultiScaleGeometricHarmonicsInterpolator(GeometricHarmonicsInterpolator):
     def fit(
         self, X: np.ndarray, y=None, store_kernel_matrix=False, **fit_params
     ) -> "MultiScaleGeometricHarmonicsInterpolator":
-        X, y = self._validate(X, y=y, ensure_min_samples=2)
+        X, y = self._validate(X, y=y, during_fit=True, ensure_min_samples=2)
 
         self._multi_scale_optimize(X, y)
         # self._precompute_aux()
@@ -509,7 +542,7 @@ class MultiScaleGeometricHarmonicsInterpolator(GeometricHarmonicsInterpolator):
         )
 
 
-class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMixin):
+class LaplacianPyramidsInterpolator(RegressorMixin, MultiOutputMixin, BaseEstimator):
     """Laplacian pyramids interpolation of function values on data manifold using
     kernels with different scales.
 
@@ -571,6 +604,7 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
 
     def __init__(
         self,
+        *,  # keyword-only
         initial_epsilon: float = 10.0,
         mu: float = 2.0,
         residual_tol: Optional[float] = None,
@@ -588,7 +622,7 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
     def level_(self):
         return 0 if self._level_tracker == {} else max(self._level_tracker.keys())
 
-    def _validate(self, X, y=None, ensure_min_samples=1):
+    def _validate(self, X, y, ensure_y, ensure_min_samples):
 
         if self.residual_tol is None and not self.auto_adaptive:
             raise ValueError(
@@ -618,7 +652,7 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
         else:
             copy = False
 
-        kwargs = {
+        check_params = {
             "accept_sparse": False,
             "copy": copy,
             "force_all_finite": True,
@@ -630,16 +664,15 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
             "ensure_min_features": 1,
         }
 
-        if y is not None:
-            kwargs["multi_output"] = True
-            kwargs["y_numeric"] = True
-            X, y = check_X_y(X, y, **kwargs)
-
+        if ensure_y:
+            check_params["multi_output"] = True
+            check_params["y_numeric"] = True
+            X, y = self._validate_data(X=X, y=y, reset=True, **check_params)
             if y.ndim == 1:
                 y = y[:, np.newaxis]
 
         else:
-            X = check_array(X, **kwargs)
+            X = check_array(X, **check_params)
 
         return X, y
 
@@ -812,7 +845,7 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
 
     def _laplacian_pyramid(self, X, y):
 
-        func_approx = np.zeros_like(y, dtype=np.float)
+        func_approx = np.zeros_like(y, dtype=float)
 
         # compute once and only apply eval
         distance_matrix = self._distance_matrix(X)
@@ -922,7 +955,7 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
         LaplacianPyramidsInterpolator
             self
         """
-        self.X_, y = self._validate(X, y, ensure_min_samples=2)
+        self.X_, y = self._validate(X, y, ensure_y=True, ensure_min_samples=2)
         self._setup()
 
         self.n_targets_ = y.shape[1]
@@ -944,7 +977,7 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
             Predicted function values of shape `(n_samples, n_targets_)`.
         """
 
-        X, _ = self._validate(X)
+        X, _ = self._validate(X, y=None, ensure_y=False, ensure_min_samples=1)
 
         check_is_fitted(self)
 
@@ -966,9 +999,8 @@ class LaplacianPyramidsInterpolator(BaseEstimator, RegressorMixin, MultiOutputMi
 
         return y_hat
 
-    def plot_eps_vs_residual(self) -> None:
-        """Plot residuals versus kernel scales (epsilon) from model fit.
-        """
+    def plot_eps_vs_residual(self) -> None:  # pragma: no cover
+        """Plot residuals versus kernel scales (epsilon) from model fit."""
 
         check_is_fitted(self)
 
