@@ -1,17 +1,17 @@
 import abc
-from typing import Any, Callable, Dict, Optional, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import numexpr as ne
 import numpy as np
 import pandas as pd
 import scipy.sparse
 import scipy.spatial
-from scipy.special import xlogy
 from sklearn.gaussian_process.kernels import Kernel
 from sklearn.preprocessing import normalize
 from sklearn.utils import check_scalar
 
 from datafold.pcfold.distance import compute_distance_matrix
+from datafold.pcfold.timeseries.accessor import TSCAccessor
 from datafold.utils.general import (
     df_type_and_indices_from,
     diagmat_dot_mat,
@@ -60,8 +60,9 @@ def _symmetric_matrix_division(
     vec: np.ndarray,
     vec_right: Optional[np.ndarray] = None,
     scalar: float = 1.0,
-) -> np.ndarray:
-    r"""Symmetric division, which often appears in kernels.
+    value_zero_division: Union[str, float] = "raise",
+) -> Union[np.ndarray, scipy.sparse.csr_matrix,]:
+    r"""Symmetric division, often appearing in kernels.
 
     .. math::
         \frac{M_{i, j}}{a v^(l)_i v^(r)_j}
@@ -70,8 +71,8 @@ def _symmetric_matrix_division(
     (left and right) vector elements :math:`v` and scalar :math:`a`.
 
     .. warning::
-        The function is in-place and may therefore overwrite the matrix. Make a copy
-        beforehand if the old values are still required.
+        The implementation is in-place and can overwrites the input matrix. Make a copy
+        beforehand if the matrix values are still required.
 
     Parameters
     ----------
@@ -84,7 +85,7 @@ def _symmetric_matrix_division(
     vec
         Vector of shape `(n_rows,)` in the denominator (Note, the reciprocal
         is is internal of the function).
-        
+
     vec_right
         Vector of shape `(n_columns,)`. If matrix is non-square or matrix input is
         not symmetric, then this input is required. If None, it is set
@@ -95,7 +96,7 @@ def _symmetric_matrix_division(
 
     Returns
     -------
-    numpy.ndarray
+
     """
 
     if matrix.ndim != 2:
@@ -103,15 +104,36 @@ def _symmetric_matrix_division(
 
     if matrix.shape[0] != matrix.shape[1] and vec_right is None:
         raise ValueError(
-            "If 'matrix' has rectangular shape, then 'vec_right' must be provided."
+            "If 'matrix' is non-square, then 'vec_right' must be provided."
         )
 
-    vec_inv_left = np.reciprocal(vec.astype(np.float64))
+    vec = vec.astype(float)
+
+    if (vec == 0.0).any():
+        if value_zero_division == "raise":
+            raise ZeroDivisionError(
+                f"Encountered zero values in division in {(vec == 0).sum()} points."
+            )
+        else:
+            # division results into 'nan' without raising a ZeroDivisionWarning. The
+            # nan values will be replaced later
+            vec[vec == 0.0] = np.nan
+
+    vec_inv_left = np.reciprocal(vec)
 
     if vec_right is None:
         vec_inv_right = vec_inv_left.view()
     else:
-        vec_inv_right = np.reciprocal(vec_right.astype(np.float64))
+        vec_right = vec_right.astype(float)
+        if (vec_right == 0.0).any():
+            if value_zero_division == "raise":
+                raise ZeroDivisionError(
+                    f"Encountered zero values in division in {(vec == 0).sum()}"
+                )
+            else:
+                vec_right[vec_right == 0.0] = np.inf
+
+        vec_inv_right = np.reciprocal(vec_right.astype(float))
 
     if vec_inv_left.ndim != 1 or vec_inv_left.shape[0] != matrix.shape[0]:
         raise ValueError(
@@ -138,22 +160,34 @@ def _symmetric_matrix_division(
 
         # The zeros are removed in the matrix multiplication, but because 'matrix' is
         # usually a distance matrix we need to preserve the "true zeros"!
-        matrix.data[matrix.data == 0] = np.nan
+        matrix.data[matrix.data == 0] = np.inf
         matrix = left_inv_diag_sparse @ matrix @ right_inv_diag_sparse
-        matrix.data[np.isnan(matrix.data)] = 0
+
+        matrix.data[np.isinf(matrix.data)] = 0
+
+        # this imposes precedence order
+        #    --> np.inf/np.nan -> np.nan
+        # i.e. for cases with 0/0, set 'value_zero_division'
+        if isinstance(value_zero_division, (int, float)):
+            matrix.data[np.isnan(matrix.data)] = value_zero_division
+
     else:
-        # Solves efficiently:
+        # This computes efficiently:
         # np.diag(1/vector_elements) @ matrix @ np.diag(1/vector_elements)
         matrix = diagmat_dot_mat(vec_inv_left, matrix, out=matrix)
         matrix = mat_dot_diagmat(matrix, vec_inv_right, out=matrix)
 
+        if isinstance(value_zero_division, (int, float)):
+            matrix[np.isnan(matrix)] = value_zero_division
+
     # sparse and dense
     if vec_right is None:
         matrix = remove_numeric_noise_symmetric_matrix(matrix)
 
     if scalar != 1.0:
-        scalar = 1 / scalar
+        scalar = 1.0 / scalar
         matrix = np.multiply(matrix, scalar, out=matrix)
+
     return matrix
 
 
@@ -163,14 +197,14 @@ def _conjugate_stochastic_kernel_matrix(
     r"""Conjugate transformation to obtain symmetric (conjugate) kernel matrix with same
     spectrum properties.
 
-    In Rabin et al. :cite:`rabin_heterogeneous_2012` (Eq. 3.1) states that \
+    Rabin et al. :cite:`rabin_heterogeneous_2012` states in equation Eq. 3.1 \
     (notation adapted):
 
     .. math::
         P = D^{-1} K
 
-    where :math:`D^{-1}` is the standard row normalization. Eq. 3.3 shows that matrix \
-    :math:`P` has a similar matrix with
+    the standard row normalization. Eq. 3.3 shows that matrix :math:`P` has a similar
+    matrix with
 
     .. math::
        A = D^{1/2} P D^{-1/2}
@@ -218,12 +252,16 @@ def _conjugate_stochastic_kernel_matrix(
         # to np.ndarray in case it is depricated format np.matrix
         left_vec = left_vec.A1
 
+    if left_vec.dtype.kind != "f":
+        left_vec = left_vec.astype(float)
+
     left_vec = np.sqrt(left_vec, out=left_vec)
 
     kernel_matrix = _symmetric_matrix_division(
         kernel_matrix, vec=left_vec, vec_right=None
     )
 
+    # This is D^{-1/2} in sparse matrix form.
     basis_change_matrix = scipy.sparse.diags(np.reciprocal(left_vec, out=left_vec))
 
     return kernel_matrix, basis_change_matrix
@@ -292,13 +330,13 @@ def _kth_nearest_neighbor_dist(
     Parameters
     ----------
     distance_matrix
-        Matrix of shape `(n_samples_Y, n_samples_X)` to partition to find the distance of
-        the `k`-th nearest neighbor. If the matrix is sparse each point must have a
-        minimum number of `k` non-zero elements.
+        Distance matrix of shape `(n_samples_Y, n_samples_X)` from which to find the
+        `k`-th nearest neighbor and its corresponding distance to return. If the matrix is
+        sparse each point must have a minimum number of `k` neighbours (i.e. non-zero
+        elements per row).
 
     k
-        The distance of the `k`-th nearest neighbor is returned. The value must be a
-        positive integer.
+        The distance of the `k`-th nearest neighbor.
 
     Returns
     -------
@@ -325,7 +363,10 @@ def _kth_nearest_neighbor_dist(
         # sparse case
 
         def _get_kth_largest_elements_sparse(
-            data: np.ndarray, indptr: np.ndarray, row_nnz, k_neighbor: int,
+            data: np.ndarray,
+            indptr: np.ndarray,
+            row_nnz,
+            k_neighbor: int,
         ):
             dist_knn = np.zeros(len(row_nnz))
             for i in range(len(row_nnz)):
@@ -345,7 +386,10 @@ def _kth_nearest_neighbor_dist(
             )
 
         dist_knn = _get_kth_largest_elements_sparse(
-            distance_matrix.data, distance_matrix.indptr, row_nnz, k,
+            distance_matrix.data,
+            distance_matrix.indptr,
+            row_nnz,
+            k,
         )
     else:
         raise TypeError(f"type {type(distance_matrix)} not supported")
@@ -355,7 +399,7 @@ def _kth_nearest_neighbor_dist(
 
 class BaseManifoldKernel(Kernel):
     @abc.abstractmethod
-    def __call__(self, X, Y=None, dist_kwargs=None, **kernel_kwargs):
+    def __call__(self, X, Y=None, *, dist_kwargs=None, **kernel_kwargs):
         """Compute kernel matrix.
 
         If `Y=None`, then the pairwise-kernel is computed with `Y=X`. If `Y` is given,
@@ -406,6 +450,26 @@ class BaseManifoldKernel(Kernel):
         )
         return f"{self.__class__.__name__}({param_str})"
 
+    def _read_kernel_kwargs(self, attrs: Optional[List[str]], kernel_kwargs: dict):
+
+        return_values: List[Any] = []
+
+        if attrs is not None:
+            for attr in attrs:
+                return_values.append(kernel_kwargs.pop(attr, None))
+
+        if kernel_kwargs != {}:
+            raise KeyError(
+                f"kernel_kwargs.keys = {kernel_kwargs.keys()} are not " f"supported"
+            )
+
+        if len(return_values) == 0:
+            return None
+        elif len(return_values) == 1:
+            return return_values[0]
+        else:
+            return return_values
+
     @staticmethod
     def read_kernel_output(
         kernel_output: Union[Union[np.ndarray, scipy.sparse.csr_matrix], Tuple]
@@ -513,6 +577,7 @@ class PCManifoldKernel(BaseManifoldKernel):
         self,
         X: np.ndarray,
         Y: Optional[np.ndarray] = None,
+        *,
         dist_kwargs: Optional[Dict[str, object]] = None,
         **kernel_kwargs,
     ):
@@ -544,7 +609,7 @@ class PCManifoldKernel(BaseManifoldKernel):
             that can be included as `**kernel_kwargs` to a follow-up ``__call__``. Note
             that if a kernel has no such values, this is empty (i.e. not even `None` is
             returned).
-        
+
         Optional[Dict]
             If the kernel computes quantities of interest, then these quantities can be
             included in this dictionary. If this is returned, then this
@@ -576,7 +641,7 @@ class PCManifoldKernel(BaseManifoldKernel):
             that the kernel acts only on stored data, i.e. distance values with
             exactly zero (duplicates and self distance) must be stored explicitly in the
             matrix. Only large distance values exceeding a cut-off should not be stored.
-        
+
         Returns
         -------
 
@@ -597,6 +662,7 @@ class TSCManifoldKernel(BaseManifoldKernel):
         self,
         X: pd.DataFrame,
         Y: Optional[pd.DataFrame] = None,
+        *,
         dist_kwargs: Optional[Dict[str, object]] = None,
         **kernel_kwargs,
     ):
@@ -642,7 +708,7 @@ class TSCManifoldKernel(BaseManifoldKernel):
         raise NotImplementedError("base class")
 
 
-class RadialBasisKernel(PCManifoldKernel):
+class RadialBasisKernel(PCManifoldKernel, metaclass=abc.ABCMeta):
     """Abstract base class for radial basis kernels.
 
     "A radial basis function (RBF) is a real-valued function whose value depends \
@@ -664,13 +730,13 @@ class RadialBasisKernel(PCManifoldKernel):
         check_scalar(
             parameter,
             name=name,
-            target_type=(int, float, np.integer, np.floating),
-            min_val=float(np.finfo(np.float64).eps),
+            target_type=(float, np.floating, int, np.integer),
+            min_val=np.finfo(float).eps,
         )
         return float(parameter)
 
     def __call__(
-        self, X, Y=None, dist_kwargs=None, **kernel_kwargs
+        self, X, Y=None, *, dist_kwargs=None, **kernel_kwargs
     ) -> Union[np.ndarray, scipy.sparse.csr_matrix]:
         """Compute kernel matrix.
 
@@ -687,8 +753,8 @@ class RadialBasisKernel(PCManifoldKernel):
             Keyword arguments passed to the distance matrix computation. See
             :py:meth:`datafold.pcfold.compute_distance_matrix` for parameter arguments.
 
-        kernel_kwargs
-            ignored
+        **kernel_kwargs
+            None
 
         Returns
         -------
@@ -697,8 +763,7 @@ class RadialBasisKernel(PCManifoldKernel):
             specified in `dist_kwargs`, then the matrix is sparse.
         """
 
-        if kernel_kwargs != {}:
-            raise ValueError(f"invalid kwargs {kernel_kwargs}")
+        self._read_kernel_kwargs(attrs=None, kernel_kwargs=kernel_kwargs)
 
         X = np.atleast_2d(X)
 
@@ -706,7 +771,10 @@ class RadialBasisKernel(PCManifoldKernel):
             Y = np.atleast_2d(Y)
 
         distance_matrix = compute_distance_matrix(
-            X, Y, metric=self.distance_metric, **dist_kwargs or {},
+            X,
+            Y,
+            metric=self.distance_metric,
+            **dist_kwargs or {},
         )
 
         kernel_matrix = self.eval(distance_matrix)
@@ -758,24 +826,34 @@ class GaussianKernel(RadialBasisKernel):
         # or other computations...)
 
         if callable(self.epsilon):
-            self.epsilon = self.epsilon(distance_matrix)
+            if isinstance(distance_matrix, scipy.sparse.csr_matrix):
+                self.epsilon = self.epsilon(distance_matrix.data)
+            elif isinstance(distance_matrix, np.ndarray):
+                self.epsilon = self.epsilon(distance_matrix)
+            else:
+                raise TypeError(
+                    f"Invalid type: type(distance_matrix)={type(distance_matrix)}."
+                    f"Please report bug."
+                )
 
         self.epsilon = self._check_bandwidth_parameter(
             parameter=self.epsilon, name="epsilon"
         )
 
-        return _apply_kernel_function_numexpr(
+        kernel_matrix = _apply_kernel_function_numexpr(
             distance_matrix,
             expr="exp((- 1 / (2*eps)) * D)",
             expr_dict={"eps": self.epsilon},
         )
 
+        return kernel_matrix
+
 
 class MultiquadricKernel(RadialBasisKernel):
     r"""Multiquadric radial basis kernel.
 
     .. math::
-        K = \sqrt(\frac{1}{2\varepsilon} \cdot D + 1)
+        K = \sqrt(\frac{1}{2 \varepsilon} \cdot D + 1)
 
     where :math:`D` is the squared euclidean distance matrix.
 
@@ -935,39 +1013,6 @@ class QuinticKernel(RadialBasisKernel):
         return _apply_kernel_function_numexpr(distance_matrix, "D ** 5")
 
 
-class ThinPlateKernel(RadialBasisKernel):
-    r"""Thin plate radial basis kernel.
-
-    .. math::
-        K = xlogy(D^2, D)
-
-
-    where :math:`D` is the Euclidean distance matrix and argument for
-    :class:`scipy.special.xlogy`.
-
-    See also super classes :class:`RadialBasisKernel` and :class:`PCManifoldKernel`
-    for more functionality and documentation.
-    """
-
-    def __init__(self):
-        super(ThinPlateKernel, self).__init__(distance_metric="euclidean")
-
-    def eval(self, distance_matrix: np.ndarray) -> np.ndarray:
-        """Evaluate the kernel on pre-computed distance matrix.
-
-        Parameters
-        ----------
-        distance_matrix
-            Matrix of pairwise distances of shape `(n_samples_Y, n_samples_X)`.
-
-        Returns
-        -------
-        Union[np.ndarray, scipy.sparse.csr_matrix]
-            Kernel matrix of same shape and type as `distance_matrix`.
-        """
-        return xlogy(np.square(distance_matrix), distance_matrix)
-
-
 class ContinuousNNKernel(PCManifoldKernel):
     """Compute the continuous `k` nearest-neighbor adjacency graph.
 
@@ -1011,12 +1056,12 @@ class ContinuousNNKernel(PCManifoldKernel):
 
         if self.k_neighbor < 1:
             raise ValueError(
-                f"'parameter 'k_neighbor={self.k_neighbor}' must be a positive integer"
+                f"parameter 'k_neighbor={self.k_neighbor}' must be a positive integer"
             )
 
         if self.delta <= 0.0:
             raise ValueError(
-                f"'parameter 'delta={self.delta}' must be a positive float"
+                f"parrameter 'delta={self.delta}' must be a positive float"
             )
 
         super(ContinuousNNKernel, self).__init__()
@@ -1029,8 +1074,8 @@ class ContinuousNNKernel(PCManifoldKernel):
         self,
         X: np.ndarray,
         Y: Optional[np.ndarray] = None,
+        *,
         dist_kwargs: Optional[Dict] = None,
-        reference_dist_knn=None,
         **kernel_kwargs,
     ):
         """Compute (sparse) adjacency graph to describes a point neighborhood.
@@ -1048,9 +1093,10 @@ class ContinuousNNKernel(PCManifoldKernel):
             Keyword arguments passed to the internal distance matrix computation. See
             :py:meth:`datafold.pcfold.compute_distance_matrix` for parameter arguments.
 
-        reference_dist_knn
-            Distances to the `k`-th nearest neighbor for each point in `X`. The values
-            are mandatory if `Y` is not `None`.
+        **kernel_kwargs: Dict[str, object]
+            - reference_dist_knn: Optional[np.ndarray]
+                Distances to the `k`-th nearest neighbor for each point in `X`. The
+                parameter is mandatory if `Y` is not `None`.
 
         Returns
         -------
@@ -1059,18 +1105,19 @@ class ContinuousNNKernel(PCManifoldKernel):
             nearest neighbor graph.
 
         Optional[Dict[str, numpy.ndarray]]
-            For a pair-wise kernel evaluation, a Dictionary with key
-            `reference_dist_knn` with the `k`-the nearest neighbors for each point are
+            For a pair-wise kernel evaluation, a dictionary with key
+            `reference_dist_knn` with the `k`-the nearest neighbors for each point is
             returned.
         """
 
-        if kernel_kwargs != {}:
-            raise ValueError(f"invalid kwargs {kernel_kwargs}")
-
         is_pdist = Y is None
 
+        reference_dist_knn = self._read_kernel_kwargs(
+            attrs=["reference_dist_knn"], kernel_kwargs=kernel_kwargs
+        )
+
         dist_kwargs = dist_kwargs or {}
-        # minimum number of neighbors required in the sparse case!
+        # minimum number of neighbors required in sparse case!
         dist_kwargs.setdefault("kmin", self.k_neighbor)
 
         distance_matrix = compute_distance_matrix(
@@ -1090,8 +1137,7 @@ class ContinuousNNKernel(PCManifoldKernel):
         if is_pdist:
             if n_samples_Y != n_samples_X:
                 raise ValueError(
-                    "if is_pdist=True, the distance matrix must be square "
-                    "and symmetric"
+                    "If is_pdist=True, the distance matrix must be square and symmetric."
                 )
 
             if isinstance(distance_matrix, np.ndarray):
@@ -1101,12 +1147,12 @@ class ContinuousNNKernel(PCManifoldKernel):
 
             if (diagonal != 0).all():
                 raise ValueError(
-                    "if is_pdist=True, distance_matrix must have zeros on diagonal "
+                    "If is_pdist=True, distance_matrix must have zeros on diagonal."
                 )
         else:
             if reference_dist_knn is None:
                 raise ValueError(
-                    "if is_pdist=False, 'reference_dist_knn' (=None) must be provided."
+                    "If is_pdist=False, 'reference_dist_knn' (=None) must be provided."
                 )
 
             if not isinstance(reference_dist_knn, np.ndarray):
@@ -1123,7 +1169,7 @@ class ContinuousNNKernel(PCManifoldKernel):
 
             if self.k_neighbor < 1 or self.k_neighbor > n_samples_X - 1:
                 raise ValueError(
-                    "n_neighbors must be in the range 1 to number of samples"
+                    "'n_neighbors' must be in a range between 1 to the number of samples."
                 )
 
     def eval(
@@ -1168,11 +1214,11 @@ class ContinuousNNKernel(PCManifoldKernel):
 
         if isinstance(distance_factors, np.ndarray):
             kernel_matrix = scipy.sparse.csr_matrix(
-                distance_factors < self.delta, dtype=np.bool
+                distance_factors < self.delta, dtype=bool
             )
         else:
             assert isinstance(distance_factors, scipy.sparse.csr_matrix)
-            distance_factors.data = (distance_factors.data < self.delta).astype(np.bool)
+            distance_factors.data = (distance_factors.data < self.delta).astype(bool)
             distance_factors.eliminate_zeros()
             kernel_matrix = distance_factors
 
@@ -1271,13 +1317,13 @@ class DmapKernelFixed(BaseManifoldKernel):
         # is set to True, then apply the the symmetry transformation
         return self.is_stochastic and self.is_symmetric
 
-    def _normalize_sampling_density_kernel_matrix(
+    def _normalize_sampling_density(
         self,
         kernel_matrix: Union[np.ndarray, scipy.sparse.csr_matrix],
         row_sums_alpha_fit: np.ndarray,
     ) -> Tuple[Union[np.ndarray, scipy.sparse.csr_matrix], Optional[np.ndarray]]:
         """Normalize (sparse/dense) kernels with positive `alpha` value. This is also
-        referred to a 'renormalization' of sampling density. """
+        referred to a 'renormalization' of sampling density."""
 
         if row_sums_alpha_fit is None:
             assert is_symmetric_matrix(kernel_matrix)
@@ -1287,16 +1333,24 @@ class DmapKernelFixed(BaseManifoldKernel):
         row_sums = kernel_matrix.sum(axis=1)
 
         if scipy.sparse.issparse(kernel_matrix):
-            # np.matrix (deprecated but used in scipy sparse matrix) to np.ndarray
+            # np.matrix to np.ndarray
+            # (np.matrix is deprecated but still used in scipy.sparse)
             row_sums = row_sums.A1
 
         if self.alpha < 1:
+            if row_sums.dtype.kind != "f":
+                # This is required for case when 'row_sums' contains boolean or integer
+                # values; for inplace operations the type has to be the same
+                row_sums = row_sums.astype(float)
+
             row_sums_alpha = np.power(row_sums, self.alpha, out=row_sums)
         else:  # no need to power with 1
             row_sums_alpha = row_sums
 
         normalized_kernel = _symmetric_matrix_division(
-            matrix=kernel_matrix, vec=row_sums_alpha, vec_right=row_sums_alpha_fit,
+            matrix=kernel_matrix,
+            vec=row_sums_alpha,
+            vec_right=row_sums_alpha_fit,
         )
 
         if row_sums_alpha_fit is not None:
@@ -1307,7 +1361,10 @@ class DmapKernelFixed(BaseManifoldKernel):
         return normalized_kernel, row_sums_alpha
 
     def _normalize(
-        self, rbf_kernel: KernelType, row_sums_alpha_fit: np.ndarray, is_pdist: bool
+        self,
+        internal_kernel: KernelType,
+        row_sums_alpha_fit: np.ndarray,
+        is_pdist: bool,
     ):
 
         # only required for symmetric kernel, return None if not used
@@ -1321,11 +1378,8 @@ class DmapKernelFixed(BaseManifoldKernel):
 
             if self.alpha > 0:
                 # if pdist: kernel is still symmetric after this function call
-                (
-                    rbf_kernel,
-                    row_sums_alpha,
-                ) = self._normalize_sampling_density_kernel_matrix(
-                    rbf_kernel, row_sums_alpha_fit
+                (internal_kernel, row_sums_alpha,) = self._normalize_sampling_density(
+                    internal_kernel, row_sums_alpha_fit
                 )
 
             if is_pdist and self.is_symmetric_transform():
@@ -1336,24 +1390,24 @@ class DmapKernelFixed(BaseManifoldKernel):
                 #        (for cdist, there is no symmetric kernel in the first place,
                 #        because it is generally rectangular and does not include self
                 #        points)
-                rbf_kernel, basis_change_matrix = _conjugate_stochastic_kernel_matrix(
-                    rbf_kernel
-                )
+                (
+                    internal_kernel,
+                    basis_change_matrix,
+                ) = _conjugate_stochastic_kernel_matrix(internal_kernel)
             else:
-                rbf_kernel = _stochastic_kernel_matrix(rbf_kernel)
+                internal_kernel = _stochastic_kernel_matrix(internal_kernel)
 
-            assert (
+            # check that if     "is symmetric pdist" -> require basis change
+            #            else   no basis change
+            assert not (
                 (is_pdist and self.is_symmetric_transform())
-                and basis_change_matrix is not None
-            ) or (
-                not (is_pdist and self.is_symmetric_transform())
-                and basis_change_matrix is None
+                ^ (basis_change_matrix is not None)
             )
 
         if is_pdist and self.is_symmetric:
-            assert is_symmetric_matrix(rbf_kernel)
+            assert is_symmetric_matrix(internal_kernel)
 
-        return rbf_kernel, basis_change_matrix, row_sums_alpha
+        return internal_kernel, basis_change_matrix, row_sums_alpha
 
     def _validate_row_alpha_fit(self, is_pdist, row_sums_alpha_fit):
         if (
@@ -1378,7 +1432,7 @@ class DmapKernelFixed(BaseManifoldKernel):
         )
 
         if isinstance(kernel_matrix, pd.DataFrame):
-            # store indices and save into same type later
+            # store indices and cast to same type later
             _type = type(kernel_matrix)
             rows_idx, columns_idx = kernel_matrix.index, kernel_matrix.columns
             kernel_matrix = kernel_matrix.to_numpy()
@@ -1386,7 +1440,9 @@ class DmapKernelFixed(BaseManifoldKernel):
             _type, rows_idx, columns_idx = None, None, None
 
         kernel_matrix, basis_change_matrix, row_sums_alpha = self._normalize(
-            kernel_matrix, row_sums_alpha_fit=row_sums_alpha_fit, is_pdist=is_pdist,
+            kernel_matrix,
+            row_sums_alpha_fit=row_sums_alpha_fit,
+            is_pdist=is_pdist,
         )
 
         if rows_idx is not None and columns_idx is not None:
@@ -1409,9 +1465,8 @@ class DmapKernelFixed(BaseManifoldKernel):
         self,
         X: np.ndarray,
         Y: Optional[np.ndarray] = None,
+        *,
         dist_kwargs: Optional[Dict] = None,
-        internal_kernel_kwargs: Optional[Dict] = None,
-        row_sums_alpha_fit: Optional[np.ndarray] = None,
         **kernel_kwargs,
     ) -> Tuple[
         Union[np.ndarray, scipy.sparse.csr_matrix], Optional[Dict], Optional[Dict]
@@ -1431,20 +1486,20 @@ class DmapKernelFixed(BaseManifoldKernel):
             Keyword arguments passed to the internal distance matrix computation. See
             :py:meth:`datafold.pcfold.compute_distance_matrix` for parameter arguments.
 
-        internal_kernel_kwargs
-            Keyword arguments passed to the set internal kernel.
-
-        row_sums_alpha_fit
-            Row sum values during re-normalization computed during pair-wise kernel
-            computation. The parameter is mandatory for the compontent-wise kernel
-            computation and if `alpha>0`.
+        **kernel_kwargs: Dict[str, object]
+            - internal_kernel_kwargs: Optional[Dict]
+                Keyword arguments passed to the set internal kernel.
+            - row_sums_alpha_fit: Optional[np.ndarray]
+                Row sum values during re-normalization computed during pair-wise kernel
+                computation. The parameter is mandatory for the compontent-wise kernel
+                computation and if `alpha>0`.
 
         Returns
         -------
         numpy.ndarray`, `scipy.sparse.csr_matrix`
             kernel matrix (or conjugate of it) with same type and shape as
             `distance_matrix`
-        
+
         Optional[Dict[str, numpy.ndarray]]
             Row sums from re-normalization in key 'row_sums_alpha_fit', only returned for
             pairwise computations. The values are required for follow up out-of-sample
@@ -1459,6 +1514,11 @@ class DmapKernelFixed(BaseManifoldKernel):
 
         is_pdist = Y is None
 
+        internal_kernel_kwargs, row_sums_alpha_fit = self._read_kernel_kwargs(
+            attrs=["internal_kernel_kwargs", "row_sums_alpha_fit"],
+            kernel_kwargs=kernel_kwargs,
+        )
+
         kernel_output = self.internal_kernel(
             X, Y=Y, dist_kwargs=dist_kwargs or {}, **internal_kernel_kwargs or {}
         )
@@ -1600,7 +1660,7 @@ class ConeKernel(TSCManifoldKernel):
         check_scalar(
             self.zeta,
             name="zeta",
-            target_type=(int, np.integer, float, np.floating),
+            target_type=(float, np.floating, int, np.integer),
             min_val=0.0,
             max_val=1.0 - np.finfo(float).eps,
         )
@@ -1608,7 +1668,7 @@ class ConeKernel(TSCManifoldKernel):
         check_scalar(
             self.epsilon,
             name="epsilon",
-            target_type=(int, np.integer, float, np.floating),
+            target_type=(float, np.floating, int, np.integer),
             min_val=np.finfo(float).eps,
             max_val=None,
         )
@@ -1635,25 +1695,16 @@ class ConeKernel(TSCManifoldKernel):
         if Y is not None:
             is_df_same_index(X, Y, check_index=False, check_column=True, handle="raise")
 
-        # checks that they are scalar:
-        X_dt = X.delta_time
-
-        if not is_float(X_dt):
-            # raises error:
-            X.tsc.check_const_time_delta()
-
-        if Y is not None:
-            Y_dt = Y.delta_time
-
-            if not is_float(X_dt):
-                # raises error:
-                Y.tsc.check_const_time_delta()
-
-            if Y_dt != X_dt:
-                raise TSCException(
-                    f"'X.delta_time={X_dt}' and 'Y.delta_time={Y_dt}' "
-                    f"have not the same time sampling."
-                )
+        # checks that if scalar, if yes returns delta_time
+        if Y is None:
+            X_dt = X.tsc.check_const_time_delta()
+        else:
+            X_dt, _ = TSCAccessor.check_equal_delta_time(
+                X,
+                Y,
+                atol=1e-15,
+                require_const=True,
+            )
 
         # return here to not compute delta_time again
         return X_dt
@@ -1678,11 +1729,8 @@ class ConeKernel(TSCManifoldKernel):
         if is_compute_distance:
             distance_matrix = np.zeros_like(cos_matrix)
 
-        # only allocate memory in first iteration, afterwards use already
-        # existing array via "out" parameter
-        diff_matrix = None
-        division = None
-        zero_mask = None
+        # define names and init as None to already to use in "out"
+        diff_matrix, denominator, zero_mask = [None] * 3
 
         for row_idx in range(cos_matrix.shape[0]):
             diff_matrix = np.subtract(X_numpy, Y_numpy[row_idx, :], out=diff_matrix)
@@ -1693,23 +1741,25 @@ class ConeKernel(TSCManifoldKernel):
                     diff_matrix, axis=1, check_finite=False
                 )
 
-            # denominator of cos: norm of time_derivative * norm_difference
-            division = np.multiply(
-                norm_timederiv_Y[row_idx], distance_matrix[row_idx, :], out=division
+            # norm of time_derivative * norm_difference
+            denominator = np.multiply(
+                norm_timederiv_Y[row_idx], distance_matrix[row_idx, :], out=denominator
             )
 
-            # nominator of cos: scalar product (time_derivative, differences)
+            # nominator: scalar product (time_derivative, differences)
             # in paper: (\xi, \omega)
             cos_matrix[row_idx, :] = np.dot(
-                timederiv_Y[row_idx, :], diff_matrix.T, out=cos_matrix[row_idx, :],
+                timederiv_Y[row_idx, :],
+                diff_matrix.T,
+                out=cos_matrix[row_idx, :],
             )
 
-            # special handling of duplicates -> division by zero leads to nan
-            zero_mask = np.equal(division, 0.0, out=zero_mask)
-            cos_matrix[row_idx, zero_mask] = 0
-            cos_matrix[row_idx, ~zero_mask] /= division[~zero_mask]
+            # special handling of (almost) duplicates -> denominator by zero leads to nan
+            zero_mask = np.less_equal(denominator, 1e-14, out=zero_mask)
+            cos_matrix[row_idx, zero_mask] = 0.0  # -> np.cos(0) = 1 later
+            cos_matrix[row_idx, ~zero_mask] /= denominator[~zero_mask]
 
-        # memory and cache efficient solving of (no intermediate memory allocations):
+        # memory and cache efficient solving with no intermediate memory allocations:
         # cos_matrix = 1 - self.zeta * np.square(np.cos(cos_matrix))
         cos_matrix = np.cos(cos_matrix, out=cos_matrix)
         cos_matrix = np.square(cos_matrix, out=cos_matrix)
@@ -1722,7 +1772,9 @@ class ConeKernel(TSCManifoldKernel):
         return cos_matrix, distance_matrix
 
     def _approx_dynflow(self, X):
-        timederiv = X.tsc.time_derivative(diff_order=1, accuracy=self.fd_accuracy)
+        timederiv = X.tsc.time_derivative(
+            scheme="center", diff_order=1, accuracy=self.fd_accuracy, shift_index=True
+        )
         norm_timederiv = df_type_and_indices_from(
             timederiv,
             values=np.linalg.norm(timederiv, axis=1),
@@ -1735,9 +1787,8 @@ class ConeKernel(TSCManifoldKernel):
         self,
         X: pd.DataFrame,
         Y: Optional[pd.DataFrame] = None,
+        *,
         dist_kwargs: Optional[Dict[str, object]] = None,
-        timederiv_X=None,
-        norm_timederiv_X=None,
         **kernel_kwargs,
     ):
         """Compute kernel matrix.
@@ -1753,17 +1804,14 @@ class ConeKernel(TSCManifoldKernel):
         dist_kwargs
             ignored `(The distance matrix is computed as part of the kernel evaluation.
             For now this can only be a dense matrix).`
-        
-        timederiv_X
-            The time derivative from a finite difference scheme. Required for a
-            component-wise evaluation.
 
-        norm_timederiv_X
-            Norm of the time derivative. Required for a component-wise evaluation.
+        **kernel_kwargs: Dict[str, object]
+            - timederiv_X
+                The time derivative from a finite difference scheme. Required for a
+                component-wise evaluation.
+            - norm_timederiv_X
+                Norm of the time derivative. Required for a component-wise evaluation.
 
-        **kernel_kwargs
-            ignored
-            
         Returns
         -------
         TSCDataFrame
@@ -1772,8 +1820,9 @@ class ConeKernel(TSCManifoldKernel):
 
         delta_time = self._validate_setting(X, Y)
 
-        if kernel_kwargs != {}:
-            raise ValueError(f"kernel_kwargs={kernel_kwargs} are not supported")
+        timederiv_X, norm_timederiv_X = self._read_kernel_kwargs(
+            attrs=["timederiv_X", "norm_timederiv_X"], kernel_kwargs=kernel_kwargs
+        )
 
         is_pdist = Y is None
 
@@ -1818,6 +1867,7 @@ class ConeKernel(TSCManifoldKernel):
                 vec=norm_timederiv_X.to_numpy().ravel(),
                 vec_right=None,
                 scalar=(delta_time ** 2) * self.epsilon,
+                value_zero_division=0,
             )
 
         else:
@@ -1866,8 +1916,11 @@ class ConeKernel(TSCManifoldKernel):
                 vec=norm_timederiv_Y.to_numpy().ravel(),
                 vec_right=norm_timederiv_X.to_numpy().ravel(),
                 scalar=(delta_time ** 2) * self.epsilon,
+                value_zero_division=0,
             )
 
+        assert np.isfinite(factor_matrix).all()
+
         kernel_matrix = _apply_kernel_function_numexpr(
             distance_matrix=distance_matrix,
             expr="exp(-1.0 * D * factor_matrix)",
@@ -1890,7 +1943,7 @@ class ConeKernel(TSCManifoldKernel):
         return kernel_matrix, ret_cdist
 
 
-class DmapKernelVariable(BaseManifoldKernel):
+class DmapKernelVariable(BaseManifoldKernel):  # pragma: no cover
     """Diffusion maps kernel with variable kernel bandwidth.
 
     .. warning::
@@ -1979,7 +2032,7 @@ class DmapKernelVariable(BaseManifoldKernel):
                 distance_matrix = distance_matrix * distance_matrix
             else:  # self.k == self.N
                 np.fill_diagonal(distance_matrix, np.nan)
-                bool_mask = ~np.diag(np.ones(nr_samples)).astype(np.bool)
+                bool_mask = ~np.diag(np.ones(nr_samples)).astype(bool)
                 distance_matrix = distance_matrix[bool_mask].reshape(
                     distance_matrix.shape[0], distance_matrix.shape[1] - 1
                 )
@@ -1987,7 +2040,7 @@ class DmapKernelVariable(BaseManifoldKernel):
             # experimental: --------------------------------------------------------------
             # paper: in var-bw paper (ref2) pdfp. 7
             # it is mentioned to IGNORE non-zero entries -- this is not detailed more.
-            # a consequence is that the NN and kernel looses symmetry, so do (K+K^T) / 2
+            # a consequence is that the NN and kernel looses symmetry, so does (K+K^T) / 2
             # This is with a cut-off rate:
             # val = 1E-2
             # distance_matrix[distance_matrix < val] = np.nan
@@ -2046,7 +2099,6 @@ class DmapKernelVariable(BaseManifoldKernel):
         kernel_eps_alpha_s = _symmetric_matrix_division(
             matrix=kernel_eps_s, vec=np.power(q_eps_s, self.alpha)
         )
-        assert is_symmetric_matrix(kernel_eps_alpha_s)
 
         return kernel_eps_alpha_s
 
@@ -2075,13 +2127,13 @@ class DmapKernelVariable(BaseManifoldKernel):
 
         return matrix_l_hat, basis_change_matrix
 
-    def __call__(
-        self, X, Y=None, kernel_kwargs=None, dist_kwargs=None,
-    ):
+    def __call__(self, X, Y=None, *, dist_kwargs=None, **kernel_kwargs):
 
         dist_kwargs = dist_kwargs or {}
         cut_off = dist_kwargs.pop("cut_off", None)
 
+        self._read_kernel_kwargs(attrs=None, kernel_kwargs=kernel_kwargs)
+
         if cut_off is not None and not np.isinf(cut_off):
             raise NotImplementedError("Handling sparsity is currently not implemented!")
 
@@ -2095,7 +2147,10 @@ class DmapKernelVariable(BaseManifoldKernel):
             )
 
         distance_matrix = compute_distance_matrix(
-            X, Y, metric="sqeuclidean", **dist_kwargs,
+            X,
+            Y,
+            metric="sqeuclidean",
+            **dist_kwargs,
         )
 
         operator_l_matrix, basis_change_matrix, rho0, rho, q0, q_eps_s = self.eval(
@@ -2132,6 +2187,9 @@ class DmapKernelVariable(BaseManifoldKernel):
         q_eps_s = self._compute_q_eps_s(kernel_eps_s, rho)
         kernel_eps_alpha_s = self._compute_kernel_eps_alpha_s(kernel_eps_s, q_eps_s)
 
+        # can be expensive
+        assert is_symmetric_matrix(kernel_eps_alpha_s)
+
         if self.is_symmetric:
             q_eps_alpha_s = kernel_eps_alpha_s.sum(axis=1)
             operator_l_matrix, basis_change_matrix = self._compute_matrix_l_conjugate(
